/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.TaggedProcessList");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");

dojo.require("com.ibm.bpm.social.Aether");

dojo.require("com.ibm.bpm.social.widgets.SmartList");
dojo.require("com.ibm.bpm.social.widgets.common._RestWidget");
dojo.require("com.ibm.bpm.social.widgets.common.BizCard");
dojo.require("com.ibm.bpm.social.widgets.dialogs.ErrorDialog");

dojo.require("com.ibm.bpm.social.util.common.Util");
dojo.require("com.ibm.bpm.social.util.common.SocialRestHandler");
dojo.require("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox");


/* TaggedProcessList - Extends SmartList. Displays posts in which the user was 
	tagged. Clicking a link will take the user to an activity stream in most 
	cases. Note that although this is named TaggedPROCESSLIST, it actually 
	doesn't extend ProcessList - it extends SmartList. This is because, due to 
	design changes, it no longer displays a list of processes. It's more like 
	a list of posts linking to processes. Note also that "tagged" is also 
	referred to as "mentioned" and this widget may be more properly labeled a 
	"Mentions" list at the end of the day.
	
	Created: October 2011
	Experts: Ben Myers @ Raleigh */
dojo.declare(
	"com.ibm.bpm.social.widgets.TaggedProcessList",
	[com.ibm.bpm.social.widgets.SmartList, com.ibm.bpm.social.widgets.common._RestWidget, dijit._Templated],
	{
	
    templateString: "",
	templatePath: dojo.moduleUrl("com.ibm.bpm.social.widgets", "templates/TaggedProcessList.html"),
	constructor: function() {
	
		//topics to subscribe to or publish to
		this.showProcessTopic = "defaultProcessListTopic";
		this.showTaskTopic = "defaultShowTaskTopic";
		this.refreshTopic = "defaultRefreshTopic";
		this.taggedProcessListSubscriptions = [];
		
		//event connectors are added to an array so they can be disconnected
		this.taggedProcessListEventConnectors = [];
		this.taggedProcessListStaticEventConnectors = []; //for the Clear All button, since it won't be destroyed on refresh
		
		//used for markTabAsNew
		this.parentTabNumber = null;
		this.parentContainer = null;
		
		//used for paging
		this.numItemsToRequest = 50;
		
		//the rest handler
		this.socialRestHandler = com.ibm.bpm.social.util.common.SocialRestHandler.getInstance();
	},
	
    postCreate: function() {
		//hide certain elements
		dojo.style(this.showMoreLink, "display", "none");
		dojo.style(this.loadingMessageSpan, "display", "none");
		dojo.style(this.loadingMessageIcon, "display", "none");
		dojo.style(this.clearListLink, "display", "none");
		
		this.inherited(arguments);
		
		//localization
    	dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		//initialize variables
		this.taggedProcessListEventConnectors = [];
		this.taggedProcessListStaticEventConnectors = [];
		this.parentTabNumber = null;
		this.parentContainer = null;
		
		//set a message to appear when the list is empty
		this.setEmptyMessage(this.messages['taggedProcessList-empty']);
		
		//set the message for the "Clear all" link and add events to it
		dojo.attr(this.clearListLink, "innerHTML", this.messages['taggedProcessList-clearList']);
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.clearListLink, 'onclick', this, "_clearAllClickEvent"));
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.clearListLink, 'keypress', this, "_clearAllKeyPressEvent"));
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.clearListLink, 'onmouseenter', dojo.hitch(this, function(evt) {
			dojo.removeClass(this.clearListIcon, "bpm-social-close-x");
			dojo.addClass(this.clearListIcon, "bpm-social-close-x-highlight");
		})));
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.clearListLink, 'onmouseleave', dojo.hitch(this, function(evt) {
			dojo.removeClass(this.clearListIcon, "bpm-social-close-x-highlight");
			dojo.addClass(this.clearListIcon, "bpm-social-close-x");
		})));
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.clearListIcon, 'onclick', this, "_clearAllClickEvent"));
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.clearListIcon, 'keypress', this, "_clearAllKeyPressEvent"));
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.clearListIcon, 'onmouseenter', dojo.hitch(this, function(evt) {
			dojo.removeClass(this.clearListIcon, "bpm-social-close-x");
			dojo.addClass(this.clearListIcon, "bpm-social-close-x-highlight");
			dojo.addClass(this.clearListLink, "linkHover");
		})));
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.clearListIcon, 'onmouseleave', dojo.hitch(this, function(evt) {
			dojo.removeClass(this.clearListIcon, "bpm-social-close-x-highlight");
			dojo.addClass(this.clearListIcon, "bpm-social-close-x");
			dojo.removeClass(this.clearListLink, "linkHover");
		})));
		
		//set the message for the "Show more" link and add events to it
		dojo.attr(this.showMoreLink, "innerHTML", this.messages['taggedProcessList-showMore']);
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.showMoreLink, 'onclick', this, "_showMoreClickEvent"));
		this.taggedProcessListStaticEventConnectors.push(dojo.connect(this.showMoreLink, 'keypress', this, "_showMoreKeyPressEvent"));
		
		//set the message for the "Loading more..." message
		dojo.attr(this.loadingMessageSpan, "innerHTML", this.messages['taggedProcessList-loadingMore']);
		
		//subscribe to refresh channel and refresh when received
		this.taggedProcessListSubscriptions.push(dojo.subscribe(this.refreshTopic, dojo.hitch(this, function(data) {
			this.refresh();
		})));
		
		this.taggedProcessListSubscriptions.push(dojo.subscribe(com.ibm.bpm.social.portal.Topics.CONNECTIONS_LOADED, dojo.hitch(this, function(data) {
			this.refresh();
		})));
    },
	
	/* addTaggedItem: Adds an item of the form "tagged you in a post". Called 
		when the referesh() REST call gets data about users tagging the 
		current user. */
	addTaggedItem: function(id, author, authorId, processId, processName, text, date) {
		//format the date if it is valid
		if(typeof(date)=='undefined' || date==null || date=='') {
			date = null;
		}
		else {
			date = com.ibm.bpm.social.util.common.Util.formatDate(date);
		}
		
		if(processName==null || processName.replace(/^\s\s*/, '').replace(/\s\s*$/, '') == "") {
			processName = this.messages["taggedProcessList-defaultProcess"].replace("${id}", processId);
		}
		
		this.addItem(
		//params - parameters for event handlers
		{	'id': id, 
			'type': "tagged", 
			'processId': processId, 
			'processName': processName
		},
		//createParams - parameters for item creation
		{	'author': author, 
			'authorId': authorId, 
			'text': text, 
			'date': date
		});
	},
	
	/* addCollaborationItem: Adds an item of the form "invited you to 
		collaborate". Called when the referesh() REST call gets data about 
		users wanting to collaborate with the current user. */
	addCollaborationItem: function(id, author, authorId, processId, processName, taskId, taskName, text, date) {
		//format the date if it is valid
		if(typeof(date)=='undefined' || date==null || date=='') {
			date = null;
		}
		else {
			date = com.ibm.bpm.social.util.common.Util.formatDate(date);
		}
		
		if(processName==null || processName.replace(/^\s\s*/, '').replace(/\s\s*$/, '') == "") {
			processName = this.messages["taggedProcessList-defaultProcess"].replace("${id}", processId);
		}
		if(taskName==null || taskName.replace(/^\s\s*/, '').replace(/\s\s*$/, '') == "") {
			taskName = this.messages["taggedProcessList-defaultTask"].replace("${id}", taskId);
		}
		
		this.addItem(
		//params - parameters for event handlers
		{	'id': id, 
			'type': "collaboration", 
			'processId': processId, 
			'processName': processName, 
			'taskId': taskId, 
			'taskName': taskName
		},
		//createParams - parameters for item creation
		{	'author': author, 
			'authorId': authorId, 
			'text': text, 
			'date': date
		});
	},
	
	/* _onItemAdded: Called whenever an item is added. */
	_onItemAdded: function(index, li, params) {
		this.inherited(arguments);
		
		//display the "Clear All" option
		dojo.style(this.clearListLink, "display", "");
		dojo.style(this.clearListIcon, "display", "");
	},
	
	/* _onItemRemoved: Hide the "Clear All" link if the list is empty. */
	_onItemRemoved: function(index, li, params) {
		if(this.numItems()==0) {
			dojo.style(this.clearListLink, "display", "none");
			dojo.style(this.clearListIcon, "display", "none");
		}
	},
	
	/* _createItem: Handles creating each item's HTML elements. */
	_createItem: function(index, li, params, createParams) {
		dojo.attr(li, 'tabindex', "0");
		//add the picture of the user that tagged you
		var imgURL = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(createParams.authorId);
		var img = dojo.create("img", {
			'class': "bpm-social-taggedprocesslist-user-img bpm-social-default-avatar-medium",
			'alt': "",
			'style': "width: 40px; height: 40px; background-image: none;", //no background image to support transparency
			'src': imgURL
		}, li);

		//if the message is "tagged you in a post"...
		if(params.type == "tagged") {
			dojo.addClass(li, "bpm-social-taggedprocesslist-tagged");

			//create the tagged notification message
			var p = dojo.create("p", {'class': "bpm-social-taggedprocesslist-message", 'innerHTML': 
				dojo.string.substitute(
					this.messages["taggedProcessList-tagNotification"], {
					'user': '<span class="bpm-social-taggedprocesslist-user">'+createParams.author+'</span>', 
					'process': '<a class="bpm-social-taggedprocesslist-process" tabindex="0" aria-label="'+params.processName+'">'+params.processName+'</a>'
				})
			}, li);
			var userName = dojo.query("span.bpm-social-taggedprocesslist-user", p)[0];
			this.taggedProcessListEventConnectors.push(dojo.connect(userName, 'onmouseenter', this, "_addTitleIfNecessary"));
			var processLink = dojo.query("a.bpm-social-taggedprocesslist-process", p)[0];
			this.taggedProcessListEventConnectors.push(dojo.connect(processLink, 'onclick', this, "_processLinkClickEvent"));
			this.taggedProcessListEventConnectors.push(dojo.connect(processLink, 'keypress', this, "_processLinkKeyPressEvent"));
			
			//create the date & time
			if(createParams.date!=null) {
				dojo.create("p", {'class': "bpm-social-taggedprocesslist-timestamp", 'innerHTML': 
					createParams.date
				}, li);
			}
			
			//create the post the user typed
			var p2 = dojo.create("p", {'class': "bpm-social-taggedprocesslist-text", 'innerHTML': 
				dojo.string.substitute(
					this.messages["taggedProcessList-quotedText"], {
					'text': createParams.text
				})
			}, li);
			
			//apply base text direction to the mentioned post
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(p2);
			
			var yourUserName = dojo.query("span.bpm-user-tag", p2)[0];
			if(yourUserName) {
				this.taggedProcessListEventConnectors.push(dojo.connect(yourUserName, 'onmouseenter', this, "_addTitleIfNecessary"));
			}
		}
		
		//if the message is "invited you to collaborate"...
		else if(params.type == "collaboration") {
			dojo.addClass(li, "bpm-social-taggedprocesslist-collaboration");
			
			//create the collaboration invitation message
			var p = dojo.create("p", {'class': "bpm-social-taggedprocesslist-message", 'innerHTML': 
				dojo.string.substitute(
					this.messages["taggedProcessList-collaborationInvitation"], {
					'user': '<span class="bpm-social-taggedprocesslist-user">'+createParams.author+'</span>', 
					'task': '<a class="bpm-social-taggedprocesslist-task" tabindex="0">'+params.taskName+'</a>', 
					'process': '<a class="bpm-social-taggedprocesslist-process" tabindex="0">'+params.processName+'</a>'
				})
			}, li);
			var userName = dojo.query("span.bpm-social-taggedprocesslist-user", p)[0];
			this.taggedProcessListEventConnectors.push(dojo.connect(userName, 'onmouseenter', this, "_addTitleIfNecessary"));
			var taskLink = dojo.query("a.bpm-social-taggedprocesslist-task", p)[0];
			this.taggedProcessListEventConnectors.push(dojo.connect(taskLink, 'onclick', this, "_taskLinkClickEvent"));
			this.taggedProcessListEventConnectors.push(dojo.connect(taskLink, 'keypress', this, "_taskLinkKeyPressEvent"));
			var processLink = dojo.query("a.bpm-social-taggedprocesslist-process", p)[0];
			this.taggedProcessListEventConnectors.push(dojo.connect(processLink, 'onclick', this, "_processLinkClickEvent"));
			this.taggedProcessListEventConnectors.push(dojo.connect(processLink, 'keypress', this, "_processLinkKeyPressEvent"));
			
			//create the date & time
			if(createParams.date!=null) {
				dojo.create("p", {'class': "bpm-social-taggedprocesslist-timestamp", 'innerHTML': 
					createParams.date
				}, li);
			}
			
			//create the post the user typed
			var p2 = dojo.create("p", {'class': "bpm-social-taggedprocesslist-text", 'innerHTML': 
				dojo.string.substitute(
					this.messages["taggedProcessList-quotedText"], {
					'text': createParams.text
				})
			}, li);
			var yourUserName = dojo.query("span.bpm-user-tag", p2)[0];
			if(yourUserName) {
				this.taggedProcessListEventConnectors.push(dojo.connect(yourUserName, 'onmouseenter', this, "_addTitleIfNecessary"));
			}
		}
		
		/** Business Card integration **/
		var user = com.ibm.bpm.social.util.common.Util.getUser(createParams.authorId);
		if(typeof(user["fullName"])=="undefined" && typeof(createParams.author)!="undefined"){
			user["fullName"] = createParams.author;
		}
		com.ibm.bpm.social.widgets.common.BizCard.wrapElement(dojo.query("span.bpm-social-taggedprocesslist-user", p)[0], user);
		com.ibm.bpm.social.widgets.common.BizCard.wrapImage(img, user);
		
		var bpmUserTags = dojo.query(".bpm-user-tag", li);
		for(var x = 0; x < bpmUserTags.length; x++){
			var objId = bpmUserTags[x].getAttribute("objId");
			if(typeof(objId)!="undefined" && objId!=null && objId!=""){
				user = com.ibm.bpm.social.util.common.Util.getUser(objId);
				com.ibm.bpm.social.widgets.common.BizCard.wrapElement(bpmUserTags[x], user);
			}
		}
		/** End Business Card integration **/
		
		//create the remove button
		var remove = dojo.create("span", {
			'style': "width: 12px; height: 12px",
			'class': "bpm-social-taggedprocesslist-closeButton bpm-social-close-x",
			'tabindex': "0",
			'title': this.messages["taggedProcessList-remove"],
			'innerHTML': '<span class="hi-contrast-icon">X</span>'
		}, li);
		this.taggedProcessListEventConnectors.push(dojo.connect(remove, 'onclick', this, "_removeClickEvent"));
		this.taggedProcessListEventConnectors.push(dojo.connect(remove, 'keypress', this, "_removeKeyPressEvent"));
		this.taggedProcessListEventConnectors.push(dojo.connect(remove, 'onmouseenter', dojo.hitch(this, function(evt) {
			dojo.addClass(remove, "hover")
			dojo.removeClass(remove, "bpm-social-close-x");
			dojo.addClass(remove, "bpm-social-close-x-highlight")
		})));
		this.taggedProcessListEventConnectors.push(dojo.connect(remove, 'onmouseleave', dojo.hitch(this, function(evt) {
			dojo.removeClass(remove, "hover");
			dojo.removeClass(remove, "bpm-social-close-x-highlight");
			dojo.addClass(remove, "bpm-social-close-x");
		})));
	},
    
	/* _addTitleIfNecessary: Adds a title to a span if its content is too long. */
	_addTitleIfNecessary: function(e) {
		var node = e.target;
		// check if the mouse is over a user tag
		if (node && (dojo.hasClass(node,'bpm-user-tag') || dojo.hasClass(node, 'bpm-social-taggedprocesslist-user'))) {
			// if the user name is clipped, provide hover help
			if (node.offsetWidth > node.parentNode.offsetWidth) {
				node.title = node.innerHTML;
			}
		}
	},
	
	/* Below are event handlers that do little other than handle basic 
		interactions with the TaggedProcessList and usually end up calling 
		_onProcessLinkClicked or _onTaskLinkClicked. */
	_processLinkClickEvent: function(evt) {
		var info = this._extractItemInfoFromEvent(evt); //from SmartList
		if(info['index']!=null && info['li']!=null) {
			this._onProcessLinkClicked(info['index'], info['li'], info['params']);
		}
	},
	_taskLinkClickEvent: function(evt) {
		var info = this._extractItemInfoFromEvent(evt); //from SmartList
		if(info['index']!=null && info['li']!=null) {
			this._onTaskLinkClicked(info['index'], info['li'], info['params']);
		}
	},
	_removeClickEvent: function(evt) {
		var info = this._extractItemInfoFromEvent(evt); //from SmartList
		if(info['index']!=null && info['li']!=null) {
			this._onRemoveClicked(info['index'], info['li'], info['params']);
		}
	},
	_clearAllClickEvent: function(evt) {
		this._onClearAllClicked();
	},
	_showMoreClickEvent: function(evt) {
		this._onShowMoreClicked();
	},
	_processLinkKeyPressEvent: function(evt) {
		if(evt.keyCode==dojo.keys.ENTER) {
			var info = this._extractItemInfoFromEvent(evt); //from SmartList
			if(info['index']!=null && info['li']!=null) {
				this._onProcessLinkClicked(info['index'], info['li'], info['params']);
			}
		}
	},
	_taskLinkKeyPressEvent: function(evt) {
		if(evt.keyCode==dojo.keys.ENTER) {
			var info = this._extractItemInfoFromEvent(evt); //from SmartList
			if(info['index']!=null && info['li']!=null) {
				this._onTaskLinkClicked(info['index'], info['li'], info['params']);
			}
		}
	},
	_removeKeyPressEvent: function(evt) {
		if(evt.keyCode==dojo.keys.ENTER) {
			var info = this._extractItemInfoFromEvent(evt); //from SmartList
			if(info['index']!=null && info['li']!=null) {
				this._onRemoveClicked(info['index'], info['li'], info['params']);
			}
		}
	},
	_clearAllKeyPressEvent: function(evt) {
		if(evt.keyCode==dojo.keys.ENTER) {
			this._onClearAllClicked();
		}
	},
	_showMoreKeyPressEvent: function(evt) {
		if(evt.keyCode==dojo.keys.ENTER) {
			this._onShowMoreClicked();
		}
	},
	
	/* _onProcessLinkClicked: Called whenever a process link is clicked. 
		Should redirect the user to the process details page. */
	_onProcessLinkClicked: function(index, li, params) {
		if(params!=null && params['processId']!=null) {
			dojo.publish(this.showProcessTopic, [{'instanceId':  params['processId']}]);
		}
	},
	
	/* _onTaskLinkClicked: Called whenever a task link is clicked. Should 
		redirect the user to the task details page if collaboration session isn't full. */
	_onTaskLinkClicked: function(index, li, params) {
		if(params!=null && params['taskId']!=null) {
			this._checkCollaborationSessionAvailablity(params['taskId']);
		}
	},
	
	/* _checkCollaborationSessionAvailablity: Get the list of collaboration participants for this task, if any. */
    _checkCollaborationSessionAvailablity: function(taskID) {
		// Response will look something like this:
		// {"participants":["UserA1","tw_admin"],"taskId":"38","taskEditor":"tw_admin","taskOwner":"tw_admin"}
		if (taskID) {
			/* /ProcessPortal/notification/roster/task/ */
			dojo.xhrGet({
				url: this.getProxyUrl()+"/ProcessPortal/notification/roster/task/"+taskID,
				handleAs: "json",
				preventCache: true,
				sync: true,
				headers: {
					"Content-Type": "application/json",
					"Accept": "application/json"
				},
				load: dojo.hitch(this, function(data, xhrArgs) { this._checkNumberOfParticipants(data, xhrArgs, taskID); }),
				error: dojo.hitch(this, this._onXhrErrors)
			});
		}
    },
	
	
	/* _checkNumberOfParticipants: See if there aren't too many participants in this task already. */
	_checkNumberOfParticipants: function (data, xhrArgs, taskID) {
		// data will look something like this:
		// {"participants":["UserA1","tw_admin"],"taskId":"38","taskEditor":"tw_admin","taskOwner":"tw_admin"}
		// or this:
		// {"message":"NO_SESSION_FOR_TASK"}
		if (!data) {
			return;
		}
		// If there's no collab session, then data won't contain an element called 'participants',
		// and in that case we go ahead and publish the show task topic.
		var doPublishShowTaskTopic = false;
		if (data.participants) {
			var participants = data.participants;
			var userProfileObj = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
			var currentUser;
			if (userProfileObj) {
				currentUser = userProfileObj.userName;
			} 
			if (participants && (participants.length < 2||_currentUserIsAlreadyInSession(currentUser,participants))) {
				// publish the show task topic
				doPublishShowTaskTopic=true;
			} else {
				com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(this.messages.collaborationSessionFull, null /* details */, function(){
				  return;
				});
			}
		} else {
			doPublishShowTaskTopic = true;
		}
		if (doPublishShowTaskTopic) {
			// publish the show task topic
			dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, [{"taskId": taskID}]);
		}
	},
	
	
	_currentUserIsAlreadyInSession:function(currentUser, participants){
		if(currentUser&&participants){
			for(var i in participants){
				if(participants[i].userName == currentUser)
					return true;
			}
		
		}
		return false;	
	},
	
	/* _onRemoveClicked: Called whenever a remove button is clicked. Should 
		permanently remove the item from the list. */
	_onRemoveClicked: function(index, li, params) {
		this.removeItem(index);
		this.numItemsToRequest--;
		if(this.numItemsToRequest <= 0) {
			this.numItemsToRequest = 50;
		}
		
		this.socialRestHandler.deleteMentions(params.id, dojo.hitch(this, function(data, xhrArgs) {
				if(this.numItems()==0) {
					this.refresh();
				}
			}), dojo.hitch(this, this._onXhrErrors), true);
	},
	
	/* _onClearAllClicked: Called whenever the Clear All link is clicked. 
		Should permanently remove all visible items from the list. */
	_onClearAllClicked: function() {
		this.clearList();
		this.numItemsToRequest = 50;
		
		this.socialRestHandler.deleteAllMentions("", dojo.hitch(this, this._onXhrErrors), true);
	},
	
	/* _onShowMoreClicked: Called whenever the Show More link is clicked. 
		Should show additional paged items if there are any. */
	_onShowMoreClicked: function() {
		dojo.style(this.showMoreLink, "display", "none");
		dojo.style(this.loadingMessageSpan, "display", "");
		dojo.style(this.loadingMessageIcon, "display", "");
		this.numItemsToRequest += 50;
		this.refresh();
	},
	
	/* refresh: Inherited from SmartList. Updates the information displayed in 
		the widget by requesting a new set of data, clearing the list, and 
		repopulating it. */
    refresh: function() {
		
		this.socialRestHandler.getMentions(1, this.numItemsToRequest, 
			dojo.hitch(this, this._doRefresh), dojo.hitch(this, this._onXhrErrors), true);
	},
	_doRefresh: function(data, xhrArgs) {
		this.clearList();
		
		var items = data.items;
		var hasMoreItems = data.more;
		
		for(var i=0; i<items.length; i++) {
			var item = items[i];
			var actorData = com.ibm.bpm.social.util.common.Util.getUser(item.actor.id);
			var processId = null;
			var processName = this.messages['taggedProcessList-unknownProcess'];
			if(typeof(item.ibmbpmext)!='undefined') {
				//take the substring in case the process begins with "2072." or something similar
				processId = item.ibmbpmext.processId.substring(item.ibmbpmext.processId.indexOf('.')+1);
				processName = item.ibmbpmext.processName;
			}
			if(item.verb=="POST") {
				this.addTaggedItem(
					item.object.id, 
					item.actor.displayName, 
					item.actor.id, 
					processId, 
					processName, 
					item.ibmbpmext.content, 
					item.published);
			}
			else if(item.verb=="INVITE") {
				var taskId = null;
				var taskName = this.messages['taggedProcessList-unknownTask'];
				if(typeof(item.ibmbpmext)!='undefined') {
					//take the substring in case the process begins with "2078." or something similar
					taskId = item.ibmbpmext.taskId.substring(item.ibmbpmext.taskId.indexOf('.')+1);
					taskName = item.ibmbpmext.taskName;
				}
				this.addCollaborationItem(
					item.object.id, 
					item.actor.displayName, 
					item.actor.id, 
					processId, 
					processName, 
					taskId, 
					taskName, 
					item.ibmbpmext.content, 
					item.published);
			}
		}
		
		//show the "Show More" link if more items exist
		if(hasMoreItems) {
			dojo.style(this.showMoreLink, "display", "");
		}
		else {
			dojo.style(this.showMoreLink, "display", "none");
		}
		dojo.style(this.loadingMessageSpan, "display", "none");
		dojo.style(this.loadingMessageIcon, "display", "none");
	},
	
	/* claimControlofTab: Gives the TaggedProcessList a ProtalSidebar for it 
		to call markTabAsNew on in the future. Should be set once after the 
		widget is created. There is no harm in choosing not to use this 
		function, other than *NEW* messages won't be displayed. */
	claimControlOfTab: function(container, tabNumber) {
		this.parentContainer = container;
		this.parentTabNumber = tabNumber;
	},
	
	/* markTabAsNew: If the TaggedProcessList is contained by a PortalSidebar, 
		it will add a small *NEW* message to the tab containing it. This 
		allows the current user to see when new messages have arrived even if 
		not currently viewing the tab. */
	markTabAsNew: function() {
		if(this.parentContainer!=null && this.parentTabNumber!=null) {
			this.parentContainer.setTabTag(this.parentTabNumber, this.messages["taggedProcessList-newMessages"], true);
		}
	},
	
    _onXhrErrors: function(data, xhrArgs) {
    	console.debug([data, xhrArgs]);
    },
	
	/* clearList: Make sure to clear extra event connectors. */
	clearList: function() {
		this.inherited(arguments);
		
		//disconnect all dojo event connectors
		dojo.forEach(this.taggedProcessListEventConnectors, function(connector) {
			dojo.disconnect(connector);
		});
		
		//hide the "Clear All" option
		dojo.style(this.clearListLink, "display", "none");
		dojo.style(this.clearListIcon, "display", "none");
		dojo.style(this.showMoreLink, "display", "none");
	},
	
	destroy: function(finalize) {
		//disconnect all dojo event connectors
		dojo.forEach(this.taggedProcessListStaticEventConnectors, function(connector) {
			dojo.disconnect(connector);
		});
		
		for(var i=0; i<this.taggedProcessListSubscriptions.length; i++) {
			dojo.unsubscribe(this.taggedProcessListSubscriptions[i]);
		}
		this.inherited(arguments);
	}
	
});


