/* 
   Licensed Materials - Property of IBM
   5725-C96
   Copyright IBM Corporation 2011. All Rights Reserved.
   US Government Users Restricted Rights- Use, duplication or disclosure
   restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.bpm.social.widgets.diagram.ActivityOwnerTooltip");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.TooltipDialog");

dojo.require("com.ibm.bpm.social.util.common.Util");
dojo.require("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox");

dojo.declare(
	"com.ibm.bpm.social.widgets.diagram.ActivityOwnerTooltip",
	[dijit._Widget, dijit._Templated],
	{
	
    templateString: "",
	templatePath: dojo.moduleUrl("com.ibm.bpm.social.widgets.diagram", "templates/ActivityOwnerTooltip.html"),
	tkiid: null,
	livenames: null,	
	popup: null,
	userName: null,
	firstName: null,
	displayName: null,
	avgHours: 0,
	completed: 0,
	title: null,
	taskEmailAddress: null,
	phoneNumber: null,
	taskDetails: null,
	chatString: null,
	includeTaskDetails: true,
	tabInd: 0,
	
    postCreate: function() {
		dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
		this.messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
		
		this.livenames = {};
		
		if(this.displayName){
			this.displayNameField.innerHTML = this.displayName;
			this.firstName = this.displayName.split(" ")[0];
			//configure BTD on the displayNameField
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.displayNameField);
		}
		
		if(this.title) {
			this.titleField.innerHTML = this.title;
			//configure BTD on the title field
			com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setElementBTD(this.titleField);
		}
		
		if(this.phoneNumber) {
			this.phoneField.innerHTML = this.phoneNumber;
		}
		
		if(this.taskEmailAddress) {
			this.emailField.innerHTML = '<a href="mailto:'+this.taskEmailAddress+'">'+this.taskEmailAddress+'</a>';
		}
		
		if(this.includeTaskDetails){
			if(this.completed > 0){
				var data = {
						'userName': this.firstName,
						'average': this.avgHours,
						'completed': this.completed
				};
				this.taskDetailsField.innerHTML = dojo.replace(this.messages.expertsTooltip, data, /\$\{([^\}]+)\}/g);
			} else {
				this.taskDetailsField.innerHTML = this.messages.expertNoTaskData.replace("${user}", this.firstName);
			}
		}
		
		this._updateAvatar();
    	this._updateSametime();
	},
	
	_updateAvatar: function(){
		
		var avatarURL = com.ibm.bpm.social.util.common.Util.getUserAvatarURL(this.userName);
 		this.img.innerHTML = "";
		var img = document.createElement('img');
		img.alt = this.fullName;
		img.className = 'bpm-social-default-avatar-small';
		img.src = avatarURL;
		this.img.appendChild(img);
	},
	
	_updateSametime: function(){
		if(window.sametime){
			this.displayNameField.innerHTML = "";

			this.livenames[this.userName] = window.sametime.LiveName({'userId': this.userName, 'disableClicks': true, 'disableHoverBizCard': true},dojo.create("span", {}, this.displayNameField));
			this.livenames[this.userName].activate();
			
			var link = dojo.create("a", {
				'href': 'javascript:void(0)',
				'username': this.userName,
				'class': 'chatLink',
				'innerHTML': this.messages.expertsChat
			}, this.chatStringField);
			dojo.connect(link, 'onclick', this, this._handleChat);

			var span = dojo.create("span", {
				'innerHTML': this.messages.expertsChatCoda.replace("${user}", this.displayName)
			}, this.chatStringField);
			
			this._updateStatus();
			this.refresh = setInterval(dojo.hitch(this, this._updateStatus), 2000);
		}
	},
	
	_updateStatus: function(){
		dojo.removeClass(this.domNode, "online");
		dojo.removeClass(this.domNode, "offline");
		
		if(this.livenames && this.livenames[this.userName]){
			switch(this.livenames[this.userName].currentTextStyle){
				case "stproxy_liveNameAvailableText":
				case "stproxy_liveNameAwayText":
				case "stproxy_liveNameInMeetingText":
					dojo.addClass(this.domNode, "online");
					this.presenceStatus = "online";
					break;
				case "stproxy_liveNameOfflineText":
				case "stproxy_liveNameDndText":
				default:
					dojo.addClass(this.domNode, "offline");
					this.presenceStatus = "offline";
					break;
			}
		} else {
			dojo.style(this.chatStringField, "display", "none");
		}
		
	},
	
	_handleChat: function(e){
		var uname = e.target.getAttribute('username');
		if(this.livenames[uname]){
			this.livenames[uname].startChat();
		}
	},
	
	//Configure fix for accessibility defect 88867: Need to manually manage tab index and key events
	//Will Return a boolean determining whether the dialog should be closed.
	handleKeyEvent: function(evt) {
		console.log("gettingcalled");
		var keypressed = evt.charCode ? evt.charCode : evt.keyCode;
		if(keypressed == dojo.keys.TAB) {
			console.log("tabkey!");
			if(evt.shiftKey) {
				console.log("decrement");
				this.tabInd--;
			} else {
				console.log("increment");
				this.tabInd++;
			}
			console.log("tabind = " + this.tabInd);
		}
		if (this.tabInd == -1 ||
			this.tabInd == 2 ||
			(this.tabInd == 1 && this.emailField.innerHTML == "")) {
			//close the dialog, because tab order is outside the dialog
			console.log("close");
			return true;
		}
		//otherwise let the tab order do its natural thing
		return false;
	},
				
	destroy: function() {
		if(this.refresh){
			clearInterval(this.refresh);
		}
		
		this.inherited(arguments);
	}

});
