// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5724-L01
// Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT


dojo.provide("com.ibm.bpm.social.widgets.notifications");

dojo.require("dijit._Widget");
dojo.require("dojox.cometd");
dojo.require("com.ibm.bpm.social.Aether");
dojo.require("com.ibm.bpm.social.portal.Topics");
dojo.require("com.ibm.bpm.social.widgets.dialogs.ErrorDialog");

dojo.declare("com.ibm.bpm.social.widgets.notifications", dijit._Widget, {

bpm_social_messages : {},
connectedToBayeuxServer : false,
currentUser:undefined,
connectionEstablished : false,

constructor: function() {	
	dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");
	this.bpm_social_messages = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");
},
	
postCreate: function(){
	window["connectionEstablished"]= this.connectionEstablished;
	dojo.addOnUnload(function(){dojox.cometd.disconnect();});
	this.connectToBayeuxServer();
},

connectToBayeuxServer: function () {
	console.debug("connecting to  Bayeux server");
	
	// configure bayeux server url and other client options
	dojox.cometd.configure(
			{
				url      : "/ProcessPortal/notification/cometd"			}
	);
	
	// disable the websocket transport
	dojox.cometd.websocketEnabled = false;	

	// add a listener to the /meta/connect channel to inform us of connection events
	dojox.cometd.addListener("/meta/connect", handleOnMetaConnectMessage);
	
	// connect to bayuex server
	dojox.cometd.handshake();
	
},

handleOnMetaConnectMessage: function (message) {
	var wasPreviouslyConnected = this.connectedToBayeuxServer;
	this.connectedToBayeuxServer = ( message.successful === true );
	if(!wasPreviouslyConnected && this.connectedToBayeuxServer) {	
		this.handleConnectionSucceeded(message);
	} else if (wasPreviouslyConnected && !this.connectedToBayeuxServer) {
		this.handleConnectionBroken();
	}
},

handleConnectionSucceeded: function (message) {
	this.connectionEstablished = true;
	console.debug("handleConnectionSucceeded, setting windows variable cometd");
	window["cometDForCollab"]= dojox.cometd;
	window["collaborationContainerDiv"]= dojo.byId("processPortalCollaborationBanner");
	console.debug("handleConnectionSucceeded, calling subscribe to Channels");
	this.subscribeToChannels();
},

getUserInfo: function (){
	//var user = com.ibm.bpm.social.util.common.Util.getCurrentUser();//TO-DO need to use Kam's code
	var user = null;
		dojo.xhrGet({
			url: '/rest/bpm/wle/v1/user/current?parts=all',
			handleAs: 'json',
			preventCache: true,
			headers: {
				"Content-Type": "application/json", 
				"Accept": "application/json"
			},
			sync: true,
			load:  function(data, xhrArgs){
				userData = data.data;
			},
			error: dojo.hitch(this, handleFailedRestConnection)
		});
	return userData;
},
	
handleConnectionBroken:function (message) {
	this.connectionEstablished = false;
	console.debug(message);
},

handleFailedRestConnection:function (message){
	console.debug(message);
},

subscribeToChannels:function (){
	console.group("subscribeToChannels");
	if(!this.currentUser){
		var userData = this.getUserInfo();
		 this.currentUser = userData;
	}
	console.debug("Subscribing to channel /bpm/notifications/" + userData.userName);
	dojox.cometd.subscribe('/bpm/notifications/'+userData.userName,this.handleNotificationMessage);
	var memberships = userData.memberships;
	if(memberships){
	    //batch the subscriptions so that we are not making multiple HTTP requests for each group subscription
		dojox.cometd.batch(function()	{
			for(var i =0;i<memberships.length;i++){
				console.debug("Subscribing to channel /bpm/notifications/" + userData.memberships[i]);
				dojox.cometd.subscribe('/bpm/notifications/'+userData.memberships[i], this.handleNotificationMessage);
			}
		});		
	}
	console.groupEnd();
},

handleNotificationMessage: function (incomingMessage){	
	data = incomingMessage.data;
	console.log("Incoming Notification Message: ", data);
	if(data) {
		switch (data.kind) {
		case "TASK_RESOURCE_ASSIGNED":
			this.handle_TASK_RESOURCE_ASSIGNED(data);
			break;
		case "TASK_COLLABORATION_INVITE":
		    this.handle_TASK_COLLABORATION_INVITE(data);
			break;
		case "PROCESS_COMMENT_TAGGED":
		    this.handle_PROCESS_COMMENT_TAGGED(data);
			break;
		default:
			break;
		}
	}
	
},

handle_TASK_RESOURCE_ASSIGNED: function (data) {
	if(!data.task.resource) return;
	
	var shouldDisplayPopup = true;
	
	switch (data.task.resource.type) {
	case "ROLE":
		this.notificationMessage = dojo.string.substitute(this.bpm_social_messages.groupReceivedNewTask,{'groupname': data.task.resource.displayName});
		var usr = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
		if(usr != null && usr.userPreferences != null && usr.userPreferences['Social Notification New Task To My Group']=="false") {
			shouldDisplayPopup = false;
		}
		break;
	case "USER":
		this.notificationMessage = this.bpm_social_messages.userReceivedNewTask;
		var usr = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
		if(usr != null && usr.userPreferences != null && usr.userPreferences['Social Notification New Task To Me']=="false") {
			shouldDisplayPopup = false;
		}
		break;
	default:
		break;
	}
	
	if(shouldDisplayPopup) {
		var link_id = this.getUniqueID("launch_task_action");
		var notificationPopUpText = "<div>${0} <a id='"+link_id+"' href='#' dojoattachevent='onclick: _handleActionClick'>${1}</a></div>";
		var aetherParams = {};
		aetherParams.message = dojo.string.substitute(notificationPopUpText, [this.notificationMessage, data.task.name]);
		aetherParams.containerDivId = "messageNotificationDiv";
		aetherParams.position = "top-right";
		aetherParams.fade = true;
		aetherParams.parent = this;
		aetherParams.displayCloseIcon = false;
		aetherParams.eventParam = data;
		this.message = new com.ibm.bpm.social.Aether(aetherParams);
		this.message.fireEvent = dojo.hitch(this,fireEvent);	
	}
};
			
handle_TASK_COLLABORATION_INVITE:function (data) {
	var invitorInfo = com.ibm.bpm.social.util.common.Util.getUser(data.resource.id);
	var task_link_id = this.getUniqueID("launch_task_action");
	var process_link_id = this.getUniqueID("launch_process_action");
	var notificationPopUpText = dojo.string.substitute(this.bpm_social_messages.collaborationInviteMessage, {'taskName': data.task.name, 'taskLinkID': task_link_id, 'processLinkID': process_link_id,'actor':data.resource.displayName, 'processName':data.process.instance.name, 'messageBody':data.task.collaboration.message}),
	var aetherParams = {};
	aetherParams.message = notificationPopUpText;
	aetherParams.containerDivId = "messageNotificationDiv";
	aetherParams.position = "top-right";
	aetherParams.fade = true;
	aetherParams.parent = this;
	aetherParams.displayCloseIcon = true;
	if(invitorInfo&&invitorInfo.image)
		aetherParames.img_src = invitorInfo.image;
	aetherParams.eventParam = data;
	this.message = new com.ibm.bpm.social.Aether(aetherParams);
	this.message.fireEvent = dojo.hitch(this,fireEvent);	
},

handle_PROCESS_COMMENT_TAGGED:function (data) {
	//publish event to notify mentioned panel for refresh
	dojo.publish(com.ibm.bpm.social.portal.Topics.REFRESH_MENTIONS_LIST,[]);
	
	var invitorInfo = com.ibm.bpm.social.util.common.Util.getUser(data.process.instance.comment.author.id);
	console.log(invitorInfo);
	
	if(true) { //TODO: logic
		var link_id = this.getUniqueID("launch_process_action");
		var notificationPopUpText = dojo.string.substitute(this.bpm_social_messages.UserMentionedMessage, {'taskName': data.process.instance.name, 'linkID': link_id, 'actor':data.process.instance.comment.author.name, 'messageBody':data.process.instance.comment.text}),
		var aetherParams = {};
		aetherParams.message = notificationPopUpText;
		aetherParams.containerDivId = "messageNotificationDiv";
		aetherParams.position = "top-right";
		aetherParams.fade = true;
		aetherParams.parent = this;
		aetherParams.displayCloseIcon = true;
		if(invitorInfo&&invitorInfo.image)
				aetherParames.img_src = invitorInfo.image;
		aetherParams.eventParam = data;
		this.message = new com.ibm.bpm.social.Aether(aetherParams);
		this.message.fireEvent = dojo.hitch(this,fireEvent);
	}
},

fireEvent:function (link_id,param){//custom override	
	var eventType = link_id.split("::")[0];
	
	switch (eventType) {
	case "launch_task_action":
		this.checkCollaborationSessionAvailablity(param.task.id);
		break;
	case "launch_process_action":
		this.dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, [{"instanceId": param.process.instance.id}]);		
		break;
	default:
		break;
	}
},
	
	/* checkCollaborationSessionAvailablity: Get the list of collaboration participants for this task, if any. */
checkCollaborationSessionAvailablity:function (taskID) {
	// Response will look something like this:
	// {"participants":["UserA1","tw_admin"],"taskId":"38","taskEditor":"tw_admin","taskOwner":"tw_admin"}
	if (taskID) {
		/* /ProcessPortal/notification/roster/task/ */
		dojo.xhrGet({
			//url: this.getProxyUrl()+"/ProcessPortal/notification/roster/task/"+taskID,
			url: "/ProcessPortal/notification/roster/task/"+taskID,
			handleAs: "json",
			preventCache: true,
			sync: true,
			headers: {
				"Content-Type": "application/json",
				"Accept": "application/json"
			},
			load: dojo.hitch(this, function(data, xhrArgs) { this.checkNumberOfParticipants(data, xhrArgs, taskID); }),
			error: dojo.hitch(this, this.handleFailedRestConnection)
		});
	}
},
	
	
	/* checkNumberOfParticipants: See if there aren't too many participants in this task already. */
checkNumberOfParticipants: function (data, xhrArgs, taskID) {
	// data will look something like this:
	// {"participants":["UserA1","tw_admin"],"taskId":"38","taskEditor":"tw_admin","taskOwner":"tw_admin"}
	// or this:
	// {"message":"NO_SESSION_FOR_TASK"}
	if (!data) {
		return;
	}
	// If there's no collab session, then data won't contain an element called 'participants',
	// and in that case we go ahead and publish the show task topic.
	var doPublishShowTaskTopic = false;
	if (data.participants) {
		var participants = data.participants;
		if (participants && participants.length < 2) {
			// publish the show task topic
			doPublishShowTaskTopic=true;
		} else {
			com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(this.bpm_social_messages.collaborationSessionFull, null /* details */, function(){
			  return;
			});
		}
	} else {
		doPublishShowTaskTopic = true;
	}
	if (doPublishShowTaskTopic) {
		// publish the show task topic
		dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, [{"taskId": taskID}]);
	}
},

getUniqueID: function (prefix){
	var currentTime = new Date();
	return prefix+"::"+currentTime.getTime();
}
}


