// StartCollaborationDialog.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.widgets.task.util.StartCollaborationDialog");

dojo.require("dijit.Dialog");
dojo.require("com.ibm.bpm.social.widgets.task.util.Constants");
dojo.require("com.ibm.bpm.social.widgets.task.util.DateTime");
dojo.require("com.ibm.bpm.social.util.common.TaskRestHandler");
dojo.require("com.ibm.bpm.social.widgets.task.util.ErrorHandler");
dojo.require("com.ibm.bpm.social.util.ui.userchooser.UserChooser");
dojo.require("com.ibm.bpm.social.util.ui.userchooser.models.BPMModel");
dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("com.ibm.bpm.social.util.baseTextDirection.BidiTextBox");

dojo.require("com.ibm.bspace.common.util.ui.Dialog");
dojo.require("com.ibm.bspace.common.util.ui.Button");
dojo.require("com.ibm.bspace.common.util.ui.Helper");
dojo.require("com.ibm.bspace.common.util.ui.FilteringSelect");


dojo.requireLocalization("com.ibm.bpm.social", "bpmTask");

dojo.declare("com.ibm.bpm.social.widgets.task.util.StartCollaborationDialog", null, {

  constructor: function(parameters){
    this._restHandler = com.ibm.bpm.social.util.common.TaskRestHandler.getInstance();
    this._errorHandler = com.ibm.bpm.social.widgets.task.util.ErrorHandler.getInstance();

    this._messageField = null;

    var task = parameters.task;

    if (typeof task.TKIID === "string") {
      this.mapping = com.ibm.bpm.social.widgets.task.util.StartCollaborationDialog.queryMapping;
    }
    else {
      this.mapping = com.ibm.bpm.social.widgets.task.util.StartCollaborationDialog.itemMapping;
    }
    this._availableActions = parameters.availableActions;
    this._tkiid = task[this.mapping["TKIID"]];
    this._taskName = task[this.mapping["NAME"]];
    this._taskDisplayName = task[this.mapping["DISPLAY_NAME"]];
    this._processInstanceName = task[this.mapping["PI_NAME"]];
    this._piid = task[this.mapping["PIID"]];

    this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
    this._connects = [];
    this._wrappedElements = [];

    if (task[this.mapping["OWNER"]]) {
      this._currentOwner = {
        value: task[this.mapping["OWNER"]],
        type: "user"
      };
    }
    else if (task[this.mapping["ASSIGNED_TO_ROLE_DISPLAY_NAME"]]) {
      this._currentOwner = {
        value: task[this.mapping["ASSIGNED_TO_ROLE_DISPLAY_NAME"]],
        type: "group"
      };
    }
    else if (task[this.mapping["ASSIGNED_TO_ROLE"]]) {
      // From Process Portal: determine group display name
      // Participant Names are in the form <PGName>_<'S'|'T'>_<PG-UUID>.<Snapshot-UUID>
      // So we are just using the last two '_'s to pull the PG name
      var assignedToRole = task[this.mapping["ASSIGNED_TO_ROLE"]];
      if (assignedToRole.match(".*_[TS]_[\\da-f-]+\\.[\\da-f-]+")) {
        var roleName = assignedToRole;
        var displayName = roleName.substring(0, roleName.lastIndexOf("_"));
        displayName = displayName.substring(0, displayName.lastIndexOf("_"));
        assignedToRole = displayName;
      }
      this._currentOwner = {
        value: assignedToRole,
        type: "group"
      };
    }
    else {
      var error = new Error("Task does not have an owner or a group. ID: " + task[this.mapping["TKIID"]]);
      com.ibm.bpm.social.util.common.Log.consoleError(error.message);
      throw error;
    }
  },

  /**
   * a value in a field has changed, check validation to enable/disable Invite push button
   */
  _onValueChanged: function _onValueChanged_StartCollaborationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var valid = true;
    valid = (this.userChooser.chooserInput.value.length < 0 || this.userChooser.getChoice() !== null);

    if (this._saveButton) {
      com.ibm.bspace.common.util.ui.Button.setDisabled(this._saveButton, !valid);
    }

    com.ibm.bpm.social.util.common.Log.logExit(2, "valid=" + valid);
  },

  /**
   * 
   * @type {void}
   */
  destroy: function destroy_StartCollaborationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    while (this._connects && this._connects.length > 1) {
      dojo.disconnect(this._connects.pop());
    }
    if (this._wrappedElements.length > 0) {
      com.ibm.bspace.common.util.ui.Helper.unwrapElementList(this._wrappedElements.length > 0);
    }
    this._restHandler = null;
    this._errorHandler = null;
    this._messageField = null;

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /**
   * Invite a user for collaboration
   * 
   * @param userid {void} user id to the assign the task to
   * @param message {void} user id to the assign the task to
   */
  _inviteUserToCollaborate: function _inviteUserToCollaborate_StartCollaborationDialog(userid, message){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var myself = this;
    this._restHandler.inviteUserToCollaborate(this._tkiid, function(response){
      myself._currentOwner = {
        value: userid,
        type: "user"
      };
      // myself.onInvitation(userid, message);
    }, function(data, ioArgs){
      var messageParameters = {
        1: myself._tkiid
      };
      // the errorCodeMap will overrule the statusCodeMap
      var errorCodeMap = {
        "CWTBG0007E": {
          key: "KEY_TaskInvitedUserDoesNotExist",
          parameters: {
            1: userid
          }
        },
        "CWTBG0561E": {
          key: "KEY_TaskInviteNotAllowedForUserid",
          parameters: {
            1: userid
          }
        },
        "CWTBG0018E": {
          key: "KEY_TaskInviteTaskNotClaimed"
        },
        "CWTBG0048E": {
          key: "KEY_TaskInviteTaskNotFound"
        },
        "CWTBG0540E": {
          key: "KEY_TaskInviteNotAllowedForTask"
        }
      };
      // for error codes not found in the errorCodeMapm use the statusCodeMap
      var statusCodeMap = {
        404: {
          key: "KEY_TaskInviteNotAllowedForTask"
        },
        403: {
          key: "KEY_TaskInviteNotAllowedForUserid",
          parameters: {
            1: userid
          }
        },
        401: {
          key: "KEY_TaskInviteNotAllowedForUserid",
          parameters: {
            1: userid
          }
        },
        400: {
          key: "KEY_TaskInviteNotAllowedForTask"
        }
      };

      myself._errorHandler.showError(data, ioArgs, statusCodeMap, errorCodeMap);
    }, userid, message);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  showDialog: function showDialog_StartCollaborationDialog(prepopulatedUseridString){
    com.ibm.bpm.social.util.common.Log.logEntry(1);
    this.dialog = new com.ibm.bspace.common.util.ui.Dialog({
      templateString: '<div dojoAttachPoint="content" ></div>',
      title: this._nlsResources["task_dialog_startCollaboration_title"], // "Collaborate with another user on this task",
      style: "min-width: 50%"
    });

    // make this similar to the task modification dialog   TODO: discuss and maybe do a seperate class
    dojo.addClass(this.dialog.content, "bpm-task-modify-task-dialog");

    var content = this.dialog.content;

    // Invited User section
    this._createInvitedUserSection(content, prepopulatedUseridString);

    // Invitation Message
    var taskLabel = this._taskDisplayName;
    if (!taskLabel || taskLabel === "") {
      // second chance: try name rather than displayName
      taskLabel = this._taskName;
    }
    //add the Bidi UCC characters to this segment of string
    taskLabel = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(taskLabel);
    var instanceName = this._processInstanceName;
    if (!instanceName || instanceName === "") {
      // fallback: use instance id rather than instance name
      instanceName = this._piid;
    }
    //add the Bidi UCC characters to this segment of string
    instanceName = com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.applyUCCToText(instanceName);

    var msg = dojo.string.substitute(this._nlsResources["task_dialog_startCollaboration_messageDefault"], {
      1: taskLabel,
      2: instanceName
    });
    this._createMessageSection(content, msg);

    // Create save button
    var saveButton = document.createElement("button");
    saveButton.innerHTML = this._nlsResources["task_dialog_startCollaboration_saveButton"]; // "Invite"
    saveButton.title = this._nlsResources["task_dialog_startCollaboration_saveButtonTooltip"];
    if (document.body.dir == 'rtl') {
      dojo.style(saveButton, "float", "left");
    }
    else {
      dojo.style(saveButton, "float", "right");
    }

    saveButton.onclick = dojo.hitch(this, this.onSave);
    var buttonSection = document.createElement("div");
    dojo.style(buttonSection, "background", "white");
    buttonSection.appendChild(saveButton);
    this._saveButton = com.ibm.bspace.common.util.ui.Button.wrapElement(saveButton);

    content.appendChild(buttonSection);

    // User Chooser value controls whether Invite button is enabled/disabled, initialize:
    this._onValueChanged();

    this.dialog.show();
    com.ibm.bpm.social.util.common.Log.logExit(1);
  },

  _createMessageSection: function _createMessageSection_StartCollaborationDialog(parentNode, initialMsg){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var messageSection = document.createElement("div");
    this._styleSection(messageSection);
    var messageLabel = document.createElement("div");
    messageLabel.innerHTML = this._nlsResources["task_dialog_startCollaboration_messageSectionTitle"]; // "Message";
    messageLabel.title = this._nlsResources["task_dialog_startCollaboration_messageSectionTooltip"];
    this._styleSectionTitle(messageLabel);
    messageSection.appendChild(messageLabel);
    // label for A11y
    var label = dojo.create("label", {
      "for": parentNode.id + "_MessageTextField",
      "style": "display: none"
    }, messageSection, "last");
    label.innerHTML = this._nlsResources["task_dialog_startCollaboration_messageSectionTitle"]; // reuse the header here (en: "Message") 
    this._messageField = dojo.create("textarea", {
      id: parentNode.id + "_MessageTextField",
      rows: "3",
      cols: "85",
      value: initialMsg,
      title: this._nlsResources["task_dialog_startCollaboration_messageSectionTooltip"]
    }, messageSection, "last");
	
	//configure base text direction here:
	com.ibm.bpm.social.util.baseTextDirection.BidiTextBox.setInputElemBTD(this._messageField);

    parentNode.appendChild(messageSection);
    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  /*
   * Returns null if user to be invited is null or the empty string.
   */
  _getMessageText: function _getMessageText_StartCollaborationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    var messageText = "[no message specified]"; // User should never see this - no NLS necessary
    if (this._messageField) {
      messageText = this._messageField.value;
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return messageText;
  },

  /*
   * Returns null if user to be invited is null or the empty string.
   */
  _getInvitedUserid: function _getInvitedUserid_StartCollaborationDialog(){
    com.ibm.bpm.social.util.common.Log.logEntry(2);
    // TODO: will this work with groups, too?
    var choice = null;
    if (this.userChooser) {
      choice = this.userChooser.getChoice();
      if (choice && (!this._currentOwner || choice.userName !== this._currentOwner.value)) {
        return {
          value: choice.userName,
          type: "user"
        };
      }
    }
    com.ibm.bpm.social.util.common.Log.logExit(2);
    return choice;
  },

  _createInvitedUserSection: function _createInvitedUserSection_StartCollaborationDialog(parentNode,
    prepopulatedUseridString){
    com.ibm.bpm.social.util.common.Log.logEntry(2);


    var invitedUserSection = document.createElement("div");
    this._styleSection(invitedUserSection);

    var invitedUserLabel = document.createElement("div");
    invitedUserLabel.innerHTML = this._nlsResources["task_dialog_startCollaboration_invitedUserSectionTitle"]; // "Select the ~ ";
    invitedUserLabel.title = this._nlsResources["task_dialog_startCollaboration_invitedUserSectionTooltip"];
    this._styleSectionTitle(invitedUserLabel);
    invitedUserSection.appendChild(invitedUserLabel);

    // Create UserChooser 
    var ucModel = new com.ibm.bpm.social.util.ui.userchooser.models.BPMModel({
      includeGroups: false,
      includeUsers: true
    });

    var myself = this;
    myself._ucClickedFirstTime = false;
    var args = {
      searchMinInputLength: 0,
      ucModel: ucModel,
      _onUCFocus: function(e){
        if (!myself._ucClickedFirstTime) {
          myself._ucClickedFirstTime = true;
          dojo.removeClass(myself.userChooser.chooserInput, "bpm-task-modify-task-dialog_uc_help");
          myself.userChooser.chooserInput.value = "";
        }
        com.ibm.bpm.social.util.ui.userchooser.UserChooser.prototype._onUCFocus.apply(this, arguments);
      },
      _onUCBlur: function(e){
        var valid = (myself.userChooser.chooserInput.value.length < 0 || myself.userChooser.getChoice() !== null);
        if(!valid) {
          myself.userChooser.chooserInput.value = "";
          myself.userChooser._hideResults();
        }
        dojo.addClass(myself.userChooser.noUsersFoundDiv,"hidden");
      }
    };
    this.userChooser = new com.ibm.bpm.social.util.ui.userchooser.UserChooser(args, null);
    this.userChooser.onChangeHandler = dojo.hitch(this, this._onValueChanged);

    // initially show a help text in the input field 
    this.userChooser.chooserInput.value = this._nlsResources["task_dialog_startCollaboration_invitedUseridFieldHelp"];
    dojo.addClass(this.userChooser.chooserInput, "bpm-task-modify-task-dialog_uc_help");


    // Set the filter that makes sure only users eligible for being invited are offered in the user chooser
    this.userChooser.setCollabTaskidFilter(this._tkiid);

    // allow to set a string in the UserChooser field, e.g. for calling this dialog
    // from the Experts List, thereby pre-populating the selected user from the Experts list.
    if (prepopulatedUseridString) {
      if (prepopulatedUseridString.fullName) {
    	  this.userChooser._chooseItemValue(prepopulatedUseridString); 
      } else {
    	  this.userChooser.chooserInput.value = prepopulatedUseridString;
      }
      myself._ucClickedFirstTime = true;
      dojo.removeClass(myself.userChooser.chooserInput, "bpm-task-modify-task-dialog_uc_help");
    }

    invitedUserSection.appendChild(this.userChooser.domNode);

    // A11y: add label for user chooser
    var ucLabel = document.createElement("label");
    ucLabel.htmlFor = this.userChooser.inputDiv.firstChild.id;
    ucLabel.innerHTML = this._nlsResources["task_dialog_startCollaboration_invitedUseridField_label"];
    ucLabel.style.display = "none";
    invitedUserSection.appendChild(ucLabel);


    parentNode.appendChild(invitedUserSection);

    com.ibm.bpm.social.util.common.Log.logExit(2);
  },

  onInvitation: function onInvitation_StartCollaborationDialog(userid, message){},

  /**
   * 
   * @type {void}
   */
  onSave: function onSave_StartCollaborationDialog(e){
    com.ibm.bpm.social.util.common.Log.logEntry(1);

    var invitedUser = this._getInvitedUserid();
    var invitedUserid = "[not selected yet]";
    if (invitedUser) {
      invitedUserid = invitedUser.value;
    }
    var messageText = this._getMessageText();

    // alert("invitation sent to " + invitedUserid +", message text is '" + messageText + "'");

    this._inviteUserToCollaborate(invitedUserid, messageText);

    this.dialog.hide();
    this.dialog.destroy();

    com.ibm.bspace.common.util.ui.Helper.wrapElement(this.dialog.domNode);

    com.ibm.bpm.social.util.common.Log.logExit(1);
    return;
  },

  /* TODO: replace with common styles */
  _styleSection: function _styleSection_StartCollaborationDialog(node){
    dojo.style(node, "padding", "10px 0");
    dojo.style(node, "background", "white");
  },

  /* TODO: replace with common styles */
  _styleSectionTitle: function _styleSectionTitle_StartCollaborationDialog(node){
    dojo.style(node, "fontWeight", "bold");
    dojo.style(node, "padding", "0 0 10px 0");
  }
});


com.ibm.bpm.social.widgets.task.util.StartCollaborationDialog.queryMapping = {
  "TKIID": "TKIID",
  "NAME": "NAME",
  "DISPLAY_NAME": "DISPLAY_NAME",
  "PI_NAME": "PI_NAME",
  "PIID": "PIID",
  "OWNER": "OWNER",
  "ASSIGNED_TO_ROLE": "ASSIGNED_TO_ROLE",
  "ASSIGNED_TO_ROLE_DISPLAY_NAME": "ASSIGNED_TO_ROLE_DISPLAY_NAME"
};

com.ibm.bpm.social.widgets.task.util.StartCollaborationDialog.itemMapping = {
  "TKIID": "tkiid",
  "NAME": "name",
  "DISPLAY_NAME": "displayName",
  "PI_NAME": "processInstanceName",
  "PIID": "piid",
  "OWNER": "owner",
  "ASSIGNED_TO_ROLE": "assignedToDisplayName",
  "ASSIGNED_TO_ROLE_DISPLAY_NAME": "assignedToDisplayName"
};
